(function() {
  var DEFAULTS, RedisFs, connectToRedis, fs, log, redis, temp, uuid, _;
  var __slice = Array.prototype.slice, __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  exports.version = '0.1.0';
  exports.redisfs = function(options) {
    return new exports.RedisFs(options);
  };
  _ = require('underscore');
  fs = require('fs');
  temp = require('temp');
  uuid = require('node-uuid');
  redis = require('redis');
  log = console.log;
  DEFAULTS = Object.seal({
    deleteKey: true,
    deleteFile: true,
    encoding: 'utf8',
    namespace: 'redisfs',
    prefix: 'redisfs-'
  });
  RedisFs = (function() {
    function RedisFs(options, keys, files) {
      if (options == null) {
        options = {};
      }
      this.keys = keys != null ? keys : [];
      this.files = files != null ? files : [];
      this.config = _.extend(_.clone(DEFAULTS), options);
      this.redis = options.redis || connectToRedis(options);
    }
    RedisFs.prototype.file2redis = function() {
      var callback, file, options, _i;
      file = arguments[0], options = 3 <= arguments.length ? __slice.call(arguments, 1, _i = arguments.length - 1) : (_i = 1, []), callback = arguments[_i++];
      options = this.applyConfig(options);
      return fs.readFile(file, options.encoding, __bind(function(err, data) {
        if (err != null) {
          return callback(err);
        }
        this.set(options.key || this.key(), data, callback);
        if (options.deleteFile === true) {
          return this.deleteFiles([_.remove(file, this.files)]);
        }
      }, this));
    };
    RedisFs.prototype.redis2file = function() {
      var callback, key, options, _i;
      key = arguments[0], options = 3 <= arguments.length ? __slice.call(arguments, 1, _i = arguments.length - 1) : (_i = 1, []), callback = arguments[_i++];
      options = this.applyConfig(options);
      if (options.file != null) {
        return this.get(key, __bind(function(err, value) {
          if (err != null) {
            return callback(err);
          }
          this.write(options.file, value, options.encoding, callback);
          if (options.deleteKey === true) {
            return this.deleteKeys(_.remove(key, this.keys));
          }
        }, this));
      } else {
        return this.open(key, options, callback);
      }
    };
    RedisFs.prototype.cleanup = function(options) {
      var both, files, keys;
      if (options == null) {
        both = true;
      }
      both || (both = options.keys && options.files);
      keys = both ? true : options.keys || false;
      files = both ? true : options.files || false;
      if (keys) {
        this.deleteKeys();
      }
      if (files) {
        return this.deleteFiles();
      }
    };
    RedisFs.prototype.end = function(options) {
      if (options !== false) {
        this.cleanup(options);
      }
      return this.redis.quit();
    };
    RedisFs.prototype.get = function(key, callback) {
      return this.redis.get(key, __bind(function(err, value) {
        return callback(err, value);
      }, this));
    };
    RedisFs.prototype.set = function(key, value, callback) {
      return this.redis.set(key, value, __bind(function(err, reply) {
        if (err != null) {
          return callback(err);
        }
        return callback(null, {
          key: key,
          reply: reply
        });
      }, this));
    };
    RedisFs.prototype.key = function() {
      var key;
      this.keys.push(key = "" + this.config.namespace + ":" + (uuid()));
      return key;
    };
    RedisFs.prototype.open = function(key, options, callback) {
      options.file = temp.path({
        prefix: options.prefix,
        suffix: options.suffix
      });
      this.files.push(options.file);
      return this.redis2file(key, options, callback);
    };
    RedisFs.prototype.applyConfig = function(options) {
      return _.extend(_.clone(this.config), options[0]);
    };
    RedisFs.prototype.write = function(file, value, encoding, callback) {
      return fs.writeFile(file, value, encoding, __bind(function(err) {
        return callback(err, file);
      }, this));
    };
    RedisFs.prototype.deleteKeys = function(keys) {
      var key, multi, _i, _len;
      if (keys == null) {
        keys = this.keys;
      }
      if (_.isArray(keys)) {
        multi = this.redis.multi();
        for (_i = 0, _len = keys.length; _i < _len; _i++) {
          key = keys[_i];
          multi.del(key);
        }
        multi.exec();
        if (this.keys === keys) {
          return this.keys = [];
        }
      } else {
        return this.redis.del(keys);
      }
    };
    RedisFs.prototype.deleteFiles = function(files) {
      var file, _i, _len;
      if (files == null) {
        files = this.files;
      }
      for (_i = 0, _len = files.length; _i < _len; _i++) {
        file = files[_i];
        fs.unlink(file);
      }
      if (this.files === files) {
        return this.files = [];
      }
    };
    return RedisFs;
  })();
  connectToRedis = function(options) {
    var client;
    client = redis.createClient(options.port, options.host);
    if (options.database != null) {
      client.select(options.database);
    }
    return client;
  };
  _.remove = function(value, array) {
    var index, swap;
    index = array.indexOf(value);
    if (index !== -1) {
      swap = array.pop();
      if (swap !== value) {
        array[index] = swap;
      }
    }
    return value;
  };
  exports.RedisFs = RedisFs;
}).call(this);
